# Python module
""" app.py -- balayer application controler
"""

import ui, documents, clock, settings

class Balayer :
    def __init__ (self, win) :
        self.main = win
        self.docs = {}
        self.curDoc = None
        self.urlHistory = {}
        self.clock = clock.create (win.clock)

    def switchTo (self, doc) :
        nbk = self.main.documents
        self.curDoc = doc
        page = ui.gtk_notebook_page_num (nbk, doc.getDocWidget ())
        ui.gtk_notebook_set_page (nbk, page)

    def newDocument (self, url, postData=None) :
        if self.docs.has_key (url) :
            self.switchTo (self.docs[url])
        else :
            doc = documents.create (self, self.main, url, postData)
            self.docs[url] = doc
            if self.curDoc is not None :
                doc.parentUrl = self.curDoc.getRequestedUrl ()
            else :
                doc.parentUrl = None

    def createdDocument (self, doc) :
        nbk = self.main.documents
        page = ui.gtk_notebook_get_current_page (nbk)
        w = doc.getDocWidget ()
        t = doc.getTabWidget ()
        ui.gtk_notebook_append_page (nbk, w, t)
        # append_page seems to use label's text for menu, so we must change it
        # explicitly
        ui.gtk_notebook_set_menu_label_text (nbk, w, doc.getTitle ())
        if page < 0 :
            self.curDoc = doc
        else :
            ui.gtk_notebook_set_page (nbk, page)

        self.urlHistory[doc.getRequestedUrl ()] = 1
        self.updateHistory ()

        # TODO: change doc key to actual url

    def switchedDocument (self, nbk, *args) :
        self.curDoc = None
        page = ui.gtk_notebook_get_current_page (nbk)
        if page >= 0 :
            w = ui.gtk_notebook_get_nth_page (nbk, page)
            for i in self.docs.values () :
                if i.getDocWidget () == w :
                    self.curDoc = i
                    self.updateDocUrl (i.getRequestedUrl ())
                    break
        self.titleChanged (self.curDoc)

    def titleChanged (self, doc) :
        if doc is self.curDoc :
            title = settings.appName
            if doc is not None :
                title = title + " - " + doc.getTitle ()
            self.updateTitle (title)

    def deleteCurrentDocument (self) :
        if self.curDoc is not None :
            doc = self.curDoc
            doc.stopLoading ()

            nbk = self.main.documents
            page = ui.gtk_notebook_get_current_page (nbk)
            ui.gtk_notebook_remove_page (nbk, page)

            url = doc.getRequestedUrl ()
            del self.docs[url]
            self.curDoc = None
            url = doc.parentUrl
            doc.destroy ()
            del doc

            if self.docs.has_key (url) :
                self.switchTo (self.docs[url])
            else :
                # TODO: -1 -> get_num
                self.switchedDocument (nbk, None, -1, self.main)

class AppController (Balayer) :
    def __init__ (self, win) :
        Balayer.__init__ (self, win)
        self.main.connect (self)
        ui.gtk_statusbar_push (self.main.messages, 1, "")

    def updateHistory (self) :
        urls = self.urlHistory.keys ()
        urls.sort ()
        ui.gtk_combo_set_popdown_strings (self.main.docUrl, urls)
    def updateTitle (self, title) :
        ui.gtk_window_set_title (self.main.Main, title)
    def updateDocUrl (self, url) :
        self.main.setDocUrl (url)
    def updateMessage (self, txt) :
        ui.gtk_statusbar_pop (self.main.messages, 1)
        ui.gtk_statusbar_push (self.main.messages, 1, txt)

    # UI callbacks

    def requestQuit (self, *args) :
        ui.gtk_main_quit ()

    def urlKeyPressed (self, entry, event, w) :
        if event.type == ui.GDK.KEY_PRESS \
        and event.string == "\r" :
            self.requestOpen ()
            ui.gtk_signal_emit_stop_by_name (entry, "key_press_event")
            return 1
    def requestOpen (self, *args) :
        url = self.main.getDocUrl ()
        self.newDocument (url)

    def requestReload (self, *args) :
        print self.docs.keys ()

    def requestClose (self, *args) :
        self.deleteCurrentDocument ()

create = AppController

