# Python module
""" documents.py -- manage a single document, whether HTML, text or image
"""

import ui, settings, loader, html, text, image
import sys

class Document :
    def __init__ (self, app, win, url, postData=None) :
        self.app = app
        self.win = win
        self.urlRequested = url

        self.view = None
        self.docWidget = None
        self.tabWidget = ui.gtk_label_new ('')
        self.setTitle (url)

        self.load = loader.create (url, self.urlFileAvailable, postData)

    def urlFileAvailable (self) :
        self.type = self.load.type
        self.actualUrl = self.load.actualUrl
        if self.type == "text/html" :
            self.view = html.View (self)
        elif self.type == "text/plain" :
            self.view = text.View (self)
        elif self.type[:6] == "image/" :
            self.view = image.View (self)
        else :
            # TODO: error / handle externally
            pass

    def createdDocument (self) :
        self.app.createdDocument (self)
    def stopLoading (self) :
        if self.load is not None :
            self.load.stop ()
            self.load = None
    def destroy (self) :
        self.view.stopLoading ()
        ui.gtk_widget_destroy (self.docWidget)
        self.docWidget = None
        ui.gtk_widget_destroy (self.tabWidget)
        self.tabWidget = None
        self.view.destroy ()
        self.view = None

    def setTitle (self, title) :
        self.title = title
        self.truncateTitle ()
        if self.docWidget is not None :
            ui.gtk_notebook_set_menu_label_text (self.win.documents,
                self.docWidget, title)
        self.app.titleChanged (self)

    def truncateTitle (self) :
        l = len (self.title)
        n = l - settings.maxTitleLength
        if n <= 0 :
            self.truncatedTitle = self.title
        else :
            n = n+3
            low = (l-n)/2
            high = low+n
            val = self.title[0:low] + "..." + self.title[high:]
            self.truncatedTitle = val
        ui.gtk_label_set_text (self.tabWidget, self.truncatedTitle)

    def getTitle (self) :
        return self.title
    def getTruncatedTitle (self) :
        return self.truncatedTitle

    def getRequestedUrl (self) :
        return self.urlRequested
    def getDocWidget (self) :
        return self.docWidget
    def getTabWidget (self) :
        return self.tabWidget

    #def __del__ (self) :
    #    print "del document", self.title

create = Document
