# Python module
""" html.py -- handle view for HTML documents
"""

import ui, loader, urlparse

currentLink = None

class View :
    """
    Handles actions coming from the view, loads images if necessary, load main
    document.
    TODO: handle arrow movements through adjustments
    """

    def __init__ (self, doc) :
        self.doc = doc
        self.view = ui.htmlView (context = doc.win)
        doc.docWidget = self.view.htmlView
        self.html = self.view.htmlDoc
        self.finish = 1
        self.baseUrl = doc.actualUrl
        self.subdocs = []

        self.view.connect (self)
        doc.createdDocument ()
        self.view.show ()
        self.inject = ui.cschtml.begin (self.html)

        doc.load.writeTo (self.writeDoc, self.endOfDoc)

    def loadDependentUrl (self, html, url, stream, win) :
        # TODO: refuse if image loading is disabled
        docUrl = urlparse.urljoin (self.baseUrl, url)
        self.subdocs.append (SubDoc (html, stream, docUrl, self.endOfSub))
    def endOfSub (self, subDoc) :
        self.subdocs.remove (subDoc)

    def writeDoc (self, data) :
        ui.cschtml.write (self.html, self.inject, data)
    def endOfDoc (self) :
        ui.cschtml.end (self.html, self.inject, ui.cschtml.STREAM_OK)
        self.inject = None
        self.doc.load = None

        if self.finish :
            self.finish = 0
            self.documentFinished (self.html, self.view)

        deps = self.subdocs[:] # work on a copy just by security
        for i in deps :
            i.start ()

    def stopLoading (self) :
        deps = self.subdocs
        self.subdocs = []
        for i in deps :
            i.stop ()

    def destroy (self) :
        self.doc = None
        self.view = None
        ui.gtk_widget_destroy (self.html)
        self.html = None

    def setDocumentTitle (self, html, newTitle, win) :
        self.doc.setTitle (newTitle)
    def setDocumentBaseUrl (self, html, base, win) :
        self.baseUrl = base
    def documentFinished (self, html, win) :
        self.finish = 0

    def showUrl (self, html, url, win) :
        global currentLink
        currentLink = url
        if url is None : url = ""
        self.doc.app.updateMessage (url)

    def newDocument (self, html, url, win) :
        docUrl = urlparse.urljoin (self.baseUrl, url)
        self.doc.app.newDocument (docUrl)

    def submitForm (self, html, method, url, encoding, win) :
        docUrl = urlparse.urljoin (self.baseUrl, url)
        if method == "POST" :
            self.doc.app.newDocument (docUrl, encoding)
        else :
            print "submit", (method, url, encoding)

    #def __del__ (self) :
    #    print "del html.View"

class SubDoc :
    """
    loads an image or frame
    """
    def __init__ (self, html, stream, url, finishedCbk=None) :
        self.html = html
        self.stream = stream
        self.url = url
        self.finished = finishedCbk
        self.load = None

    def start (self) :
        self.load = loader.create (self.url, self.started)
    def started (self) :
        self.load.writeTo (self.writeSub, self.endSub)
    def writeSub (self, data) :
        ui.cschtml.write (self.html, self.stream, data)
    def endSub (self) :
        ui.cschtml.end (self.html, self.stream, ui.cschtml.STREAM_OK)
        self.load = None
        if self.finished is not None :
            self.finished (self)
            self.finished = None

    def stop (self) :
        if self.load is not None :
            self.load.stop ()
            self.load = None

    #def __del__ (self) :
    #    print "del subdoc", self.url

