# Python module
""" loader.py -- manage url types and corresponding loading

Is the point where to trigger: document and DNS cache, rq->act url aliasing
"""

import urllib, thread
from fcntl import fcntl
from FCNTL import F_GETFL, O_NONBLOCK, F_SETFL
from gdkinput import gdk_input_add, gdk_input_remove, GDK_INPUT_READ
from _gtk import gtk_timeout_add, gtk_timeout_remove
import settings, cache

def setNonBlocking (fd) :
    flags = fcntl (fd, F_GETFL, 0)
    fcntl (fd, F_SETFL, flags|O_NONBLOCK)

class Loader :
    def __init__ (self, url, cbk, postData=None) :
        self.urlFileAvailable = cbk
        self.origUrl = url
        self.postData = postData
        self.mark = None
        self.urlFile = None
        self.saveFile = None
        self.first = 1
        self.writeCbk = None
        self.closeCbk = None

        if self.postData is None :
            self.cached = cache.get (self.origUrl)
        else :
            self.cached = None
        if self.cached is None :
            thread.start_new_thread (self.openUrl, ())
        else :
            self.openUrl ()

    #def __del__ (self) :
    #    print "del loader", self.origUrl

    def openUrl (self) :
        f = self.cached or urllib.urlopen (self.origUrl, self.postData)
        # This makes it executed in the main thread
        gtk_timeout_add (0, self.urlOpened, (f,))

    def stop (self) :
        # TODO: stop thread and/or simply close
        self.urlFileAvailable = None

    def urlOpened (self, f) :
        """
        since no return, None is used by default and the timeout under which
        this is executed is dropped.
        """
        self.urlFile = f
        self.actualUrl = f.geturl ()
        self.fileno = f.fileno ()
        info = f.info ()
        if info is None :
            self.type = "unknown"
        else :
            self.type = info.gettype ()
            if self.cached is None :
                self.saveFile = cache.put (self.actualUrl, self.type)

        if self.urlFileAvailable is not None :
            self.urlFileAvailable ()
            self.urlFileAvailable = None

    def writeTo (self, writeCbk, closeCbk) :
        self.writeCbk = writeCbk
        self.closeCbk = closeCbk
        setNonBlocking (self.fileno)
        self.mark = gdk_input_add (self.fileno, GDK_INPUT_READ, self.input, 0)

    def input (self, n, fd, what) :
        try :
            data = self.urlFile.read (settings.loaderBytesPerBlock)
        except IOError, (errNum, errMsg) :
            data = None
        if data :
            if self.saveFile is not None :
                try :
                    self.saveFile.write (data)
                except IOError :
                    self.saveFile = None
            if self.writeCbk is not None :
                self.writeCbk (data)
        else :
            self.writeCbk = None
            self.close ()

    def close (self) :
        if self.mark is not None :
            n = self.mark
            self.mark = None
            if self.urlFile is not None :
                gdk_input_remove (n)
            else :
                gtk_timeout_remove (self.mark)
        if self.saveFile is not None :
            self.saveFile.close ()
            self.SaveFile = None
        if self.closeCbk is not None :
            self.closeCbk ()
            self.closeCbk = None
        if self.urlFile is not None :
            f = self.urlFile
            self.urlFile = None
            self.fileno = None
            f.close ()

create = Loader

