# Python module
""" safe.py -- transactionned file I/O

When manipulating file with pathname X:
    - if X exists it is the stable version to use for loading
    - if only X.old and X.new exist, a new version was written but the commit
      was not finished, so loading should rename X.new to X, then use the new
      X.
    - if only X.new exists, it cannot be decided whether the commit was
      finished, so this is considered an uncommited case, and loading will
      fail.

flags are additional letters to "w" or "r" in the open() call ("b" for
example).
"""

import os

def save (name, contents, flags='') :
    old = name+'.old'
    new = name+'.new'
    if os.path.exists (old) :
        if os.path.exists (name) :
            os.remove (old)
        else :
            os.rename (old, name)
    open (new, 'w'+flags).write (contents)
    if os.path.exists (name) :
        os.rename (name, old)
    os.rename (new, name)

def load (name, flags='') :
    if not os.path.exists (name) :
        old = name+'.old'
        new = name+'.new'
        if os.path.exists (old) and os.path.exists (new) :
            os.rename (new, name)
    return open (name, "r"+flags).read ()

