# Python module
""" settings.py -- load, hold, record and UI-change settings
"""

import os, safe, string
from types import StringType, IntType

# --- settings data

appName = "Balayer"

maxTitleLength = 16
truncTitleAtEnd = 0 # TODO: replace by:
titleTruncPos = 0 # <0 start =0 middle >0 end

clockUpdate = 1000
clockFormat = " %X " # in strftime format

loaderBytesPerBlock = 102400
cacheRecordDelay = 5 # in seconds

textFont = "7x14"

# --- $BALAYER_DIR | ~/.balayer management

def getUserDir () :
    balayerEnv = "BALAYER_DIR"
    homeEnv = "HOME"
    res = None
    if os.environ.has_key (balayerEnv) :
        d = os.environ[balayerEnv]
        if os.path.isdir (d) :
            res = d
    if res is None and os.environ.has_key (homeEnv) :
        home = os.environ[homeEnv]
        if os.path.isdir (home) :
            d = os.path.join (home, ".balayer")
            if os.path.isdir (d) :
                res = d
            else :
                try :
                    os.mkdir (d)
                    res = d
                except OSError :
                    pass
    return res

def getUserCache () :
    par = getUserDir ()
    res = None
    if par is not None :
        d = os.path.join (par, "cache")
        if os.path.isdir (d) :
            res = d
        else :
            try :
                os.mkdir (d)
                res = d
            except OSError :
                pass
    return res

# --- loading and saving

def load () :
    """
    The format of the settings file is a suite of lines of the form
    attr = expr, where expr is a python expression, that yields a value of
    the same type as the attribute settings.attr.
    """
    d = getUserDir ()
    if d is None :
        return
    file = os.path.join (d, "settings")
    try :
        lines = safe.load (file)
    except IOError :
        return
    settings = globals ()
    for line in string.split (lines, "\n") :
        if "=" not in line :
            continue
        (name, val) = string.split (line, "=", 1)
        name = string.strip (name)
        if name[0] == "#" :
            continue
        try :
            val = eval (val)
        except :
            continue
        if not settings.has_key (name) :
            continue
        oldval = settings[name]
        if type (val) <> type (oldval) :
            continue
        settings[name] = val

def save () :
    d = getUserDir ()
    if d is None :
        return
    file = os.path.join (d, "settings")
    lines = []
    settings = globals ()
    attrList = settings.keys ()
    attrList.sort ()
    for a in attrList :
        if a[:1] == "_" :
            continue
        val = settings[a]
        if type (val) not in [StringType, IntType] :
            continue
        lines.append ("%s = %s\n" % (a, repr (val)))
    try :
        safe.save (file, string.join (lines, ""))
    except IOError :
        # TODO: signal that settings could not be saved
        return

# first time loading
load ()

