# Python module
"""
support -- methods for gladepyc generated code
"""

import os, types, string
from _gtk import *
import GDK, GTK

def ConnectSignal (w, sigName, listener, fName, *extra) :
    if hasattr (listener, fName) :
        func = getattr (listener, fName)
        assert callable (func)
        gtk_signal_connect (w, sigName, func, extra)
def AfterSignal (w, sigName, listener, fName, *extra) :
    if hasattr (listener, fName) :
        func = getattr (listener, fName)
        assert callable (func)
        gtk_signal_connect_after (w, sigName, func, extra)

def NewMenuFromLabels (items) :
    m = gtk_menu_new ()
    for i in items :
        gtk_menu_append (m, gtk_menu_item_new_with_label (i))
    return m

class Context :
    def __init__ (self, tooltips=None, accelgroup=None, imagedirs=None,
    window=None, context=None) :
        if isinstance (context, Context) :
            self._tooltips = context._tooltips
            self._accelgroup = context._accelgroup
            self._imagedirs = context._imagedirs
            self._top = context._top
        else :
            if tooltips is None :
                self._tooltips = gtk_tooltips_new ()
            if accelgroup is None :
                self._accelgroup = gtk_accel_group_new ()
            if imagedirs is None :
                base = os.path.dirname (__file__)
                self._imagedirs = [base, os.path.join (base, "pixmaps")]
            if window is None :
                self._top = None
        if tooltips is not None :
            self._tooltips = tooltips
        if accelgroup is not None :
            self._accelgroup = accelgroup
        if type (imagedirs) is types.StringType :
            self._imagedirs = string.split (imagedirs, os.pathsep)
        elif imagedirs is not None :
            self._imagedirs = list (imagedirs)
        if window is not None :
            self._top = window
    def setTip (self, w, msg) :
        gtk_tooltips_set_tip (self._tooltips, w, msg, None)
    def addAccel (self, w, sigName, keyVal, modVal, flagVal) :
        gtk_widget_add_accelerator (w, sigName, self._accelgroup,
            keyVal, modVal, flagVal)
    def setContainerLabelWithAccel (self, item, text, accel=None, mask=0) :
        label = gtk_container_children (item)[0]
        key = gtk_label_parse_uline (label, text)
        if accel is None :
            accel = self._accelgroup
            mask = GDK.MOD1_MASK
            gtk_widget_add_accelerator (item, "activate_item", accel, key, mask, 0)
    def findPixmapPath (self, pixName) :
        for i in self._imagedirs :
            pixPath = os.path.join (i, pixName)
            if os.path.exists (pixPath) :
                return pixPath
        raise IOError (None, "pixmap '%s' not found" % pixName)
    def show (self) :
	if self._top is not None :
	    gtk_widget_show_all (self._top)

##BEGIN GETSET SUPPORT

def SetText (w, value) :
    gtk_text_freeze (w)
    l = gtk_text_get_length (w)
    p = gtk_editable_get_position (w)
    gtk_editable_delete_text (w, 0, l)
    gtk_editable_set_position (w, 0)
    gtk_editable_insert_text (w, value)
    l = len (value)
    if p > l : p = l
    gtk_editable_set_position (w, p)
    gtk_text_thaw (w)

def SetCList (w, value) :
    gtk_clist_freeze (w)
    gtk_clist_clear (w)
    for line in value :
        gtk_clist_append (w, line)
    gtk_clist_thaw (w)
def GetCList (w) :
    nRows = 0 # TODO
    nCols = 0 # TODO
    ret = []
    for j in xrange (nRows) :
        l = map (lambda i : gtk_clist_get_text (w, j, i), xrange (nCols))
        ret.append (l)
    return ret

def SetList (w, value) :
    n = len (gtk_container_children (w))
    gtk_list_clear_item (w, 0, n)
    itemList = []
    for i in value :
        itemList.append (gtk_list_item_new_with_label (i))
    gtk_list_append_items (w, itemList)
def GetList (w) :
    ret = []
    for i in gtk_container_children (w) :
        l = gtk_container_children (i)[0]
        ret.append (gtk_label_get (l))
    return ret

##END GETSET SUPPORT

