# Python module
""" text.py -- handle view for HTML documents
"""

import ui, settings

def setFont (text, fontName) :
    sty = ui.gtk_widget_get_style (text)
    font = ui.gdk_font_load (fontName)
    sty.font = font
    ui.gtk_widget_set_style (text, sty)

class View :
    """
    load and display text contents.
    No actions to handle here.
    TODO: handle arrow movements through adjustments
    """

    def __init__ (self, doc) :
        self.doc = doc
        self.view = ui.textView (context = doc.win)
        doc.docWidget = self.view.textView
        self.text = self.view.textDoc

        setFont (self.text, settings.textFont)

        self.view.connect (self)
        doc.createdDocument ()
        self.view.show ()

        doc.load.writeTo (self.writeDoc, self.endOfDoc)

    def writeDoc (self, data) :
        ui.gtk_text_freeze (self.text)
        i = ui.gtk_text_get_point (self.text)
        l = ui.gtk_text_get_length (self.text)
        ui.gtk_text_set_point (self.text, l)
        ui.gtk_text_insert_defaults (self.text, data)
        ui.gtk_text_set_point (self.text, i)
        ui.gtk_text_thaw (self.text)

    def endOfDoc (self) :
        self.doc.load = None

    def stopLoading (self) :
        pass

    def destroy (self) :
        self.doc = None
        self.view = None
        ui.gtk_widget_destroy (self.text)
        self.text = None

    #def __del__ (self) :
    #    print "del text.View"

